@@ -59,7 +59,7 @@ def lineno():
     """Returns the current line number in our program."""
     return inspect.currentframe().f_back.f_lineno
 
-version = __version__ = "2.1"
+version = __version__ = "2.2"
 
 tokens = [
     'NUMBER',
@@ -356,6 +356,24 @@ class CppClass(dict):
             self["doxygen"] = doxygenCommentCache
             doxygenCommentCache = ""
         self["name"] = nameStack[1]
+        
+        #Handle template classes
+        if len(nameStack) > 3 and nameStack[2].startswith("<"):
+            open_template_count = 0
+            param_separator = 0
+            found_first = False
+            i = 0
+            for elm in nameStack:
+                if '<' in elm :
+                    open_template_count += 1
+                    found_first = True
+                elif '>' in elm:
+                    open_template_count -= 1
+                if found_first and open_template_count == 0:
+                    self["name"] = "".join(nameStack[1:i + 1])
+                    break;
+                i += 1
+                
         inheritList = []
 
         if ":" in nameStack:
@@ -372,6 +390,14 @@ class CppClass(dict):
                 else:
                     tmpStack = nameStack
                     nameStack = []
+                
+                # Convert template classes to one name in the last index
+                for i in range(0, len(tmpStack)):
+                    if '<' in tmpStack[i]:
+                        tmpStack2 = tmpStack[:i-1]
+                        tmpStack2.append("".join(tmpStack[i-1:]))
+                        tmpStack = tmpStack2
+                        break
                 if len(tmpStack) == 0:
                     break;
                 elif len(tmpStack) == 1:
@@ -379,8 +405,8 @@ class CppClass(dict):
                 elif len(tmpStack) == 2:
                     tmpInheritClass["access"] = tmpStack[0]
                     tmpInheritClass["class"] = tmpStack[1]
-                elif len(tmpStack) == 3 and tmpStack[0] == "virtual":
-                    tmpInheritClass["access"] = tmpStack[1]
+                elif len(tmpStack) == 3 and "virtual" in tmpStack:
+                    tmpInheritClass["access"] = tmpStack[1] if tmpStack[1] != "virtual" else tmpStack[0]
                     tmpInheritClass["class"] = tmpStack[2]
                     tmpInheritClass["virtual"] = True
                 else:
@@ -677,7 +703,23 @@ class CppMethod( _CppMethod ):
 
         self.update( methinfo )
 
+        #Filter out initializer lists used in constructors
+        try:
+            paren_depth_counter = 0
+            for i in range(0, len(nameStack)):
+                elm = nameStack[i]
+                if elm == "(":
+                    paren_depth_counter += 1
+                if elm == ")":
+                    paren_depth_counter -=1
+                    if paren_depth_counter == 0 and nameStack[i+1] == ':':
+                        debug_print("Stripping out initializer list")
+                        nameStack = nameStack[:i+1]
+                        break
+        except: pass
+        
         paramsStack = self._params_helper1( nameStack )
+        
 
         params = []
         #See if there is a doxygen comment for the variable
@@ -1505,8 +1547,6 @@ class _CppHeader( Resolver ):
             info['defined'] = True
             self._method_body = self.braceDepth
             trace_print( 'NEW METHOD WITH BODY', self.braceDepth )
-            if not self.curClass: #Method/function defined outside class braces
-                return None
         elif stack[-1] == ';':
             info['defined'] = False
             self._method_body = None    # not a great idea to be clearing here
@@ -1547,6 +1587,9 @@ class _CppHeader( Resolver ):
             klass = name[ : name.rindex('::') ]
             name = name.split('::')[-1]
             info['class'] = klass
+            if self.classes.has_key(klass) and not self.curClass:
+                 #Class function defined outside the class
+                return None
         #    info['name'] = name
         #else: info['name'] = name
 
@@ -1617,7 +1660,9 @@ class _CppHeader( Resolver ):
                 if klass['namespace']: newMethod['path'] = klass['namespace'] + '::' + klass['name']
                 else: newMethod['path'] = klass['name']
             else: #non class functions
+                debug_print("FREE FUNCTION")
                 newMethod = CppMethod(self.nameStack, None, info)
+                self.functions.append(newMethod)
             global parseHistory
             parseHistory.append({"braceDepth": self.braceDepth, "item_type": "method", "item": newMethod})
         else:
@@ -1801,6 +1846,8 @@ class CppHeader( _CppHeader ):
         # this keeps the API compatible, TODO proper namespace for everything. 
         Resolver.CLASSES = {}
         self.classes = Resolver.CLASSES
+        #Functions that are not part of a class
+        self.functions = []
 
         self.enums = []
         self.global_enums = {}
@@ -1889,7 +1936,7 @@ class CppHeader( _CppHeader ):
                         trace_print( 'END OF STRUCT DEF' )
                         self.curStruct = None
 
-                    if self._method_body and self.braceDepth < self._method_body:
+                    if self._method_body and self.braceDepth <= self._method_body:
                         self._method_body = None; self.stack = []; self.nameStack = []; trace_print( 'FORCE CLEAR METHBODY' )
                 
                 if (tok.type == 'OPEN_PAREN'):
@@ -2023,6 +2070,9 @@ class CppHeader( _CppHeader ):
                     pass
                 else: 
                     self.evaluate_method_stack()
+            else:
+                #Free function
+                self.evaluate_method_stack()
         elif is_property_namestack(self.nameStack) and self.stack[-1] == ';':
             debug_print( "trace" )
             if self.nameStack[0] in ('class', 'struct') and len(self.stack) == 3: self.evalute_forward_decl()
@@ -2092,4 +2142,8 @@ class CppHeader( _CppHeader ):
         rtn = ""
         for className in self.classes.keys():
             rtn += "%s\n"%self.classes[className]
+        if self.functions:
+            rtn += "// functions\n"
+            for f in self.functions:
+                rtn += "%s\n"%f
         return rtn
