@@ -62,3 +62,11 @@ namespace Alpha
         };
     };
 }
+
+int sampleFreeFunction(int i)
+{
+	return i + 1;
+}
+
+int anotherFreeFunction(void);
+}
